# Kafka Topics Observer Extension
<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [alpha]  |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Aextension%2Fkafkatopicsobserver%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Aextension%2Fkafkatopicsobserver) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Aextension%2Fkafkatopicsobserver%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Aextension%2Fkafkatopicsobserver) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@MovieStoreGuy](https://www.github.com/MovieStoreGuy) |

[alpha]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#alpha
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

The Kafka topics observer extension is a [Receiver Creator](../../../receiver/receivercreator/README.md)-compatible "watch observer" that will detect and report
kafka topics in kafka cluster based on regex template. This observer watches available topics and matches them with the
provided regex. If any change in available topics matching the regex is detected, the observer updates the endpoints list.

## Configuration

The following settings are required:

- `topic_regex` regex pattern of the topic name to subscribe to.

The following settings can be optionally configured:

- `brokers` (default = localhost:9092): The list of kafka brokers
- `resolve_canonical_bootstrap_servers_only` (default = false): Whether to resolve then reverse-lookup broker IPs during startup
- `protocol_version` (default = 2.1.0): Kafka protocol version e.g. 2.0.0
- `client_id` (default = "otel-collector"): The client ID to configure the Kafka client with.
- `topics_sync_interval` (default 5s)
- `auth`
    - `plain_text`
        - `username`: The username to use.
        - `password`: The password to use
    - `sasl`
        - `username`: The username to use.
        - `password`: The password to use
        - `mechanism`: The sasl mechanism to use (SCRAM-SHA-256, SCRAM-SHA-512, AWS_MSK_IAM, AWS_MSK_IAM_OAUTHBEARER or PLAIN)
        - `aws_msk.region`: AWS Region in case of AWS_MSK_IAM or AWS_MSK_IAM_OAUTHBEARER mechanism
        - `aws_msk.broker_addr`: MSK Broker address in case of AWS_MSK_IAM mechanism
    - `tls`
        - `ca_file`: path to the CA cert. For a client this verifies the server certificate. Should
          only be used if `insecure` is set to false.
        - `cert_file`: path to the TLS cert to use for TLS required connections. Should
          only be used if `insecure` is set to false.
        - `key_file`: path to the TLS key to use for TLS required connections. Should
          only be used if `insecure` is set to false.
        - `insecure` (default = false): Disable verifying the server's certificate
          chain and host name (`InsecureSkipVerify` in the tls config)
        - `server_name_override`: ServerName indicates the name of the server requested by the client
          in order to support virtual hosting.
    - `kerberos`
        - `service_name`: Kerberos service name
        - `realm`: Kerberos realm
        - `use_keytab`: Use of keytab instead of password, if this is true, keytab file will be used instead of password
        - `username`: The Kerberos username used for authenticate with KDC
        - `password`: The Kerberos password used for authenticate with KDC
        - `config_file`: Path to Kerberos configuration. i.e /etc/krb5.conf
        - `keytab_file`: Path to keytab file. i.e /etc/security/kafka.keytab
        - `disable_fast_negotiation`: Disable PA-FX-FAST negotiation (Pre-Authentication Framework - Fast). Some common Kerberos implementations do not support PA-FX-FAST negotiation. This is set to `false` by default.
- `metadata`
  - `full` (default = true): Whether to maintain a full set of metadata. When disabled, the client does not make the initial request to broker at the startup.
  - `retry`
    - `max` (default = 3): The number of retries to get metadata
    - `backoff` (default = 250ms): How long to wait between metadata retries
